@@ -281,6 +281,8 @@ instance FunctionArgs (IO Word64)        (FA Word64)        Word64        where
 instance FunctionArgs (IO ())            (FA ())            ()            where apArgs _ _ g = g
 instance (Pos n, IsPrimitive a) =>
          FunctionArgs (IO (Vector n a))  (FA (Vector n a))  (Vector n a)  where apArgs _ _ g = g
+instance StructFields as =>
+         FunctionArgs (IO (Struct as))   (FA (Struct as))   (Struct as)   where apArgs _ _ g = g
 instance (IsType a) => 
          FunctionArgs (IO (Ptr a))       (FA (Ptr a))       (Ptr a)       where apArgs _ _ g = g
 instance FunctionArgs (IO (StablePtr a)) (FA (StablePtr a)) (StablePtr a) where apArgs _ _ g = g
@@ -439,31 +441,45 @@ type TGlobal a = CodeGenModule (Global a)
 -- Special string creators
 {-# DEPRECATED createString "use withString instead" #-}
 createString :: String -> TGlobal (Array n Word8)
-createString s = string (length s) (U.constString s)
+createString s =
+    let (cstr, n) = U.constString s
+    in string n cstr
 
 {-# DEPRECATED createStringNul "use withStringNul instead" #-}
 createStringNul :: String -> TGlobal (Array n Word8)
-createStringNul s = string (length s + 1) (U.constStringNul s)
+createStringNul s =
+    let (cstr, n) = U.constStringNul s
+    in string n cstr
 
-withString ::
-   String ->
-   (forall n. (Nat n) => Global (Array n Word8) -> CodeGenModule a) ->
-   CodeGenModule a
-withString s act =
-   let n = length s
-   in  reifyIntegral n (\tn ->
-          do arr <- string n (U.constString s)
-             act (fixArraySize tn arr))
+class WithString a where
+  withString    :: String -> (forall n . Nat n => Global (Array n Word8) -> a) -> a
+  withStringNul :: String -> (forall n . Nat n => Global (Array n Word8) -> a) -> a
 
-withStringNul ::
-   String ->
-   (forall n. (Nat n) => Global (Array n Word8) -> CodeGenModule a) ->
-   CodeGenModule a
-withStringNul s act =
-   let n = length s + 1
-   in  reifyIntegral n (\tn ->
-          do arr <- string n (U.constStringNul s)
-             act (fixArraySize tn arr))
+instance WithString (CodeGenModule a) where
+  withString s act =
+    let (cstr, n) = U.constString s
+    in reifyIntegral n (\tn ->
+       do arr <- string n cstr
+          act (fixArraySize tn arr))
+
+  withStringNul s act =
+    let (cstr, n) = U.constStringNul s
+    in reifyIntegral n (\tn ->
+       do arr <- string n cstr
+          act (fixArraySize tn arr))
+
+instance WithString (CodeGenFunction r b) where
+  withString s act =
+    let (cstr, n) = U.constString s
+    in reifyIntegral n (\tn ->
+       do arr <- liftCodeGenModule $ string n cstr
+          act (fixArraySize tn arr))
+
+  withStringNul s act =
+    let (cstr, n) = U.constStringNul s
+    in reifyIntegral n (\tn ->
+       do arr <- liftCodeGenModule $ string n cstr
+          act (fixArraySize tn arr))
 
 fixArraySize :: n -> Global (Array n a) -> Global (Array n a)
 fixArraySize _ = id
